/*
The Chinese University of Hong Kong

File    : nRF24L01P.cpp
Version : 1.0
Author  : LO Ka Wo, Stanley

Last Modification Date : 2011 August 14, 05:22am

Description :
	This library provides an interface between Arduino and nRF24L01+.
	This library is able to make nRF24L01+ operate in ShockBurst mode
	with fixed data/payload length or Dynamic Data/Payload length.
	HOWEVER, this library cannot let nRF24L01+ communicate with nRF2401
	device(older model of nRF24L01+). 
	
	You can redistribute it and/or modify it.
	
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
    DEALINGS IN THE SOFTWARE.
*/

#include "nRF24L01P.h"

nRF24L01P::nRF24L01P(int cePin,   int csnPin,
					 int clkPin,  int mosiPin,
					 int misoPin)
{
	//assign Pin number
	__csnPin  = csnPin;
	__cePin   = cePin;
	__clkPin  = clkPin;
	__misoPin = misoPin;
	__mosiPin = mosiPin;

	//set Pin mode
	pinMode(__cePin, OUTPUT);
	pinMode(__csnPin, OUTPUT);
	pinMode(__clkPin, OUTPUT);
	pinMode(__mosiPin, OUTPUT);
	pinMode(__misoPin, INPUT);

	//Initialize state of output Pins before any actions
	digitalWrite(__cePin, LOW);
	digitalWrite(__csnPin, HIGH);
	digitalWrite(__clkPin, HIGH);
	digitalWrite(__mosiPin, HIGH);
}//end nRF24L01P constructor

//DefaultShockBurstSetting Setting
void nRF24L01P::DefaultShockBurstSetting(void)
{
	PowerUP();              //Set Power Up Bit

	setMode(RX);            //Switch to Receiver Mode
	
	EnableRxDrInterrupt();  //Enable RX_DR Interrupt
	EnableTxDsInterrupt();  //Enable TX_DS Interrupt
	EnableMaxRtInterrupt();          //Enable MAX_RT Interrupt
	
	EnableCRC(1);           //Enable Internal CRC Function
	EnableAutoAck(0);       //Enable Auto Ack. Data Pip 0
	EnableAutoAck(1);       //Enable Auto Ack. Data Pip 1

	EnableDataPipe(0);      //Enable RX Address Data Pipe 0
	EnableDataPipe(1);      //Enable RX Address Data Pipe 1

	setAddressWidth(5);     //Set RX/TX Address Width

	setAutoRD(15);          //Set Retransmission Delay 4ms
	setAutoReCount(15);     //Set Retransmission Count 15

	setChannel(23);         //Set Operating Channel

	setSpeedAirRate(SpeedAirRate_2MBPS); //Set Speed Air Rate 2Mbps
	setRFPower(3);          //Set RF Power 0dBm

	FlushTX();              //Flush TX FIFO Buffer
	FlushRX();              //Flush RX FIFO Buffer
	clearStatusReg();       //Clear Status Register
}//end DefaultShockBurstSetting


void nRF24L01P::DefaultDynamicPayloadSetting(void)
{
	DefaultShockBurstSetting();     // Use Default Shock Burst Setting

	DataPipeEnableDynamicPldLen(0); // Enable Data Pipe 0 with Dynamic Payload Length
	DataPipeEnableDynamicPldLen(1); // Enable Data Pipe 1 with Dynamic Payload Length
}//end DefaultDynamicPayloadSetting


//Power Up the device
void nRF24L01P::PowerUP(void)
{
	//Write Power Up bit to nRF24L01+
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, ReadRegByte(nRF24L01P_CONFIG) | 0x02);
}//end PowerUP


//Power Down the device
void nRF24L01P::PowerDown(void)
{
	//Clear Power Up bit
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, ReadRegByte(nRF24L01P_CONFIG) & 0xFD);
}//end PowerDown


//Mask Interrupt caused by RX_DR
void nRF24L01P::DisableRxDrInterrupt(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, ReadRegByte(nRF24L01P_CONFIG) | 0x40);
}//end DisableRxDrInterrupt


//Set interrupt caused by RX_DR
void nRF24L01P::EnableRxDrInterrupt(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, ReadRegByte(nRF24L01P_CONFIG) & 0xBF);
}//end EnableRxDrInterrupt


//Mask interrupt caused by TX_DS
void nRF24L01P::DisableTxDsInterrupt(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, ReadRegByte(nRF24L01P_CONFIG) | 0x20);
}//end DisableTxDsInterrupt


//Set interrupt caused by TX_DS
void nRF24L01P::EnableTxDsInterrupt(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, ReadRegByte(nRF24L01P_CONFIG) & 0xDF);
}//end EnableTxDsInterrupt


//Mask interrupt caused by MAX_RT
void nRF24L01P::DisableMaxRtInterrupt(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, ReadRegByte(nRF24L01P_CONFIG) | 0x10);
}//end DisableMaxRtInterrupt


//Set interrupt caused by MAX_RT
void nRF24L01P::EnableMaxRtInterrupt(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, ReadRegByte(nRF24L01P_CONFIG) & 0xEF);
}//end EnableMaxRtInterrupt


//Disable Internal CRC function
void nRF24L01P::DisableCRC(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, ReadRegByte(nRF24L01P_CONFIG) & 0xF7);
}//end DisableCRC


//Enable CRC function
//scheme: 1 for 2 bytes sheme CRC
//        0 for 1 byte scheme CRC
void nRF24L01P::EnableCRC(unsigned char scheme)
{
	if(scheme == 1)
		WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, ReadRegByte(nRF24L01P_CONFIG) | 0x0C);
	else
		WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, ReadRegByte(nRF24L01P_CONFIG) & 0xFB);
}//end EnableCRC


//Enable Automatic Acknowledgement
//pipe: data pipe to be enable auto ack.
//      which is from 0 to 5
void nRF24L01P::EnableAutoAck(unsigned char pipe)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_EN_AA, ReadRegByte(nRF24L01P_EN_AA) | (0x01 << pipe));
}//end EnableAutoAck


//Disable Automatic Acknowledgement
//pipe: data pipe to be enable auto ack.
//      which is from 0 to 5
void nRF24L01P::DisableAutoAck(unsigned char pipe)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_EN_AA, ReadRegByte(nRF24L01P_EN_AA) & (~(0x01 << pipe)));
}//end DisableAutoAck


//Disable Data Pipe
//pipe: Data Pipe Number
//		which is from 0 to 5
void nRF24L01P::DisableDataPipe(unsigned char pipe)
{
	DisableRXAddr(pipe);
}//end disable data pipe


//Enable Data Pipe
//pipe: Data Pipe Number
//		which is from 0 to 5
void nRF24L01P::EnableDataPipe(unsigned char pipe)
{
	EnableRXAddr(pipe);
}//end Enable data pipe


//Disable Data Pipe
//pipe: Data Pipe Number
//		which is from 0 to 5
void nRF24L01P::DisableRXAddr(unsigned char pipe)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_EN_RXADDR, ReadRegByte(nRF24L01P_EN_RXADDR) & (~(0x01 << pipe)));
}//end DisableRXAddr


//Enable Data Pipe
//pipe: Data Pipe Number
//		which is from 0 to 5
void nRF24L01P::EnableRXAddr(unsigned char pipe)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_EN_RXADDR, ReadRegByte(nRF24L01P_EN_RXADDR) | (0x01 << pipe));
}//end EnableRXAddr


//Set Address Width
//AddrWidth: number of bytes of Address Width
//			 which is from 3 to 5 bytes
//			 default is 5 bytes
void nRF24L01P::setAddressWidth(unsigned char AddrWidth)
{
	if(AddrWidth > 2 && AddrWidth <6)
		WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_SETUP_AW, AddrWidth - 2); 
	else
		WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_SETUP_AW, 3); 
}//end setAddressWidth


//Get Address Width
//return: number of bytes of Address Width
//		  which is from 3 to 5
unsigned char nRF24L01P::getAddressWidth(void)
{
	return (ReadRegByte(nRF24L01P_SETUP_AW) & 0x3) + 2; 
}//end getAddressWidth


void nRF24L01P::setAutoRD(unsigned char AutoRD)
{
	if(AutoRD < 16)
		WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_SETUP_RETR, (ReadRegByte(nRF24L01P_SETUP_RETR) & 0x0F) | (AutoRD << 4)); 
	else
		WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_SETUP_RETR, (ReadRegByte(nRF24L01P_SETUP_RETR) & 0x0F) | (15 << 4)); 
}//end setAutoRD


void nRF24L01P::setAutoReCount(unsigned char AutoCount)
{
	if(AutoCount < 16)
		WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_SETUP_RETR, (ReadRegByte(nRF24L01P_SETUP_RETR) & 0xF0) | AutoCount); 
	else
		WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_SETUP_RETR, (ReadRegByte(nRF24L01P_SETUP_RETR) & 0xF0) | 15); 
}//end setAutoReCount


void nRF24L01P::setSpeedAirRate(unsigned char Speed)
{
	unsigned char spd;
	
	switch(Speed)
	{
		// 1 Mbps
		case 0:
			spd = 0x00;
			break;

		//250 kbps
		case 2:
			spd = 0x20;
			break;
		
		// 2 Mbps
		default:
			spd = 0x08;
			break;

	}//end switch
	
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_RF_SETUP, (ReadRegByte(nRF24L01P_RF_SETUP) & 0x07) | spd); 
}//end setSpeedAirRate


void nRF24L01P::setRFPower(unsigned char PWR)
{
	if(PWR < 4)
		WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_RF_SETUP, (ReadRegByte(nRF24L01P_RF_SETUP) & 0xF8) | (PWR << 1)); 
	else
		WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_RF_SETUP, (ReadRegByte(nRF24L01P_RF_SETUP) & 0xF8) | (3 << 1)); 
}//end setRFPower


void nRF24L01P::WriteRegByte(unsigned char reg, unsigned char value)
{
	digitalWrite(__csnPin, HIGH);
	digitalWrite(__csnPin, LOW); //Initialize SPI Communications
	SPIWriteByte(reg); //Select Register to Write
	SPIWriteByte(value); //Write Value to register
	digitalWrite(__csnPin, HIGH); //Terminate SPI Communications	
}//end WriteRegByte

unsigned char nRF24L01P::ReadRegByte(unsigned char reg)
{
	unsigned char tmp;

	digitalWrite(__csnPin, HIGH);
	digitalWrite(__csnPin, LOW); //Initialize SPI Communications
	SPIWriteByte(reg); //Select Register to read
	tmp = SPIReadByte(); //Read Value from register
	digitalWrite(__csnPin, HIGH); //Terminate SPI Communications	
	
	return tmp;
}//end ReadRegBytes


void nRF24L01P::WriteRegBuf(unsigned char reg, unsigned char *buf, char bytes)
{
	char i;

	digitalWrite(__csnPin, HIGH);
	digitalWrite(__csnPin, LOW); //Initialize SPI Communications
	SPIWriteByte(reg); //Select register to write

	//Write value to Buffer
	for(i=0; i<bytes; i++)
		SPIWriteByte(buf[i]);

	digitalWrite(__csnPin, HIGH); //Terminate SPI Communications
}//end WriteRegBuf


void nRF24L01P::ReadRegBuf(unsigned char reg, unsigned char *buf, char bytes)
{
	char i;

	digitalWrite(__csnPin, HIGH);
	digitalWrite(__csnPin, LOW); //Initialize SPI Communications
	SPIWriteByte(reg); //Select register to read from 

	//Read Buffer from a register
	for(i=0; i<bytes; i++)
		buf[i] = SPIReadByte();

	digitalWrite(__csnPin, HIGH); //Terminate SPI Communications	
}//End ReadRegBuf
	

//Set Mode
//Parameters: Mode, TX for transmitter mode
//				    RX for receiver mode
void nRF24L01P::setMode(char mode)
{
	unsigned char regTmp;

	digitalWrite(__cePin, LOW);
	
	regTmp = ReadRegByte(nRF24L01P_CONFIG);

	if(mode == TX)
		regTmp &= 0xFE;
	else
		regTmp |= 0x01;

	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_CONFIG, regTmp);
	
	if(mode == RX)
		digitalWrite(__cePin, HIGH);
	
	delayMicroseconds(130);
}//end setMode


void nRF24L01P::Transmit(unsigned char *pBuf, unsigned char plWidth)
{
	//Set CE Pin Low before any actions
	digitalWrite(__cePin, LOW);
	
	//Clear the status register and TX FIFO buffer
	//before the transmission
	clearTxDs();
	clearMaxRt();
	FlushTX();

	//Switch to transmitter mode
	setMode(TX);
	
	//Send the Payload to nRF24L01+
	WriteTXPayload(pBuf, plWidth);

	//Send out the data
	digitalWrite(__cePin, HIGH);
	delayMicroseconds(15);
	digitalWrite(__cePin, LOW);
	delayMicroseconds(15);
	
	//Wait Transmission Complete
	while(!MaxRt() && !TxDs());
}//end Transmit


void nRF24L01P::Transmit(char *pBuf, unsigned char plWidth)
{
  Transmit((unsigned char *) pBuf, plWidth);
}//end Transmit


//Write Payload Buffer
void nRF24L01P::WriteTXPayload(unsigned char *pBuf, unsigned char plWidth)
{
	WriteRegBuf(nRF24L01P_WR_TX_PLOAD, pBuf, plWidth);
}//end WriteTXPayload


//SetChannel
//Parameters: RF_Channel From 0 to 125
void nRF24L01P::setChannel(unsigned char RF_Channel)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_RF_CH, RF_Channel); 
}//end setChannel


//Return RF Channel from 0 to 125
unsigned char nRF24L01P::getChannel(void)
{
	return ReadRegByte(nRF24L01P_RF_CH);
}//end getChannel


unsigned char nRF24L01P::getStatus(void)
{
	return ReadRegByte(nRF24L01P_STATUS);
}//end getStatus


unsigned char nRF24L01P::getRXPayloadWidth(unsigned char pipe)
{
	return ReadRegByte(nRF24L01P_RX_PW_P0 + pipe);
}//end getRXPayloadWidth


void nRF24L01P::setRXPayloadWidth(unsigned char pipe, unsigned char plWidth)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_RX_PW_P0 + pipe, plWidth);
}//end setRXPayloadWidth


unsigned char nRF24L01P::getCurrentPipeNum(void)
{
	return ((getStatus() & 0x0E) >> 1);
}//end getCurrentPipeNum


void nRF24L01P::getRXPload(unsigned char *pBuf)
{
	unsigned char plWidth, pipe;

	pipe = getCurrentPipeNum();

	if(isDynamicPDEnabled())
		plWidth = ReadRegByte(nRF24L01P_R_RX_PL_WID);
	else
		plWidth = getRXPayloadWidth(pipe);

	if(plWidth > 31)
	{
		FlushRX();
		return;
	}

	ReadRegBuf(nRF24L01P_RD_RX_PLOAD, pBuf, plWidth);
	clearRxDr();
}//end getRXPload


void nRF24L01P::getRXPload(unsigned char *pBuf, unsigned char *pipe, unsigned char *plWidth)
{
	*pipe = getCurrentPipeNum();

	if(isDynamicPDEnabled())
		*plWidth = ReadRegByte(nRF24L01P_R_RX_PL_WID);
	else
		*plWidth = getRXPayloadWidth(*pipe);

	if(*plWidth > 31)
	{
		FlushRX();
		return;
	}

	ReadRegBuf(nRF24L01P_RD_RX_PLOAD, pBuf, *plWidth);
	clearRxDr();
}//end getRXPload

void nRF24L01P::getData(unsigned char *pBuf)
{
	getRXPload(pBuf);
}//end getData


void nRF24L01P::getData(char *pBuf)
{
	getRXPload((unsigned char *)pBuf);
}//end getdata


void nRF24L01P::getData(unsigned char *pBuf, unsigned char *pipe, unsigned char *plWidth)
{
	getRXPload(pBuf, pipe, plWidth);
}//end getData


void nRF24L01P::getData(char *pBuf, unsigned char *pipe, unsigned char *plWidth)
{
	getRXPload((unsigned char *) pBuf, pipe, plWidth);
}//end getData


unsigned char nRF24L01P::getFIFO(void)
{
	return ReadRegByte(nRF24L01P_FIFO_STATUS);
}//end getFIFO


void nRF24L01P::setTXAddress(unsigned char *pBuf, unsigned char addrWidth)
{
	WriteRegBuf(nRF24L01P_WRITE_REG + nRF24L01P_TX_ADDR, pBuf, addrWidth);
}//end setTXAddress


void nRF24L01P::setRXAddress(unsigned char *pBuf, unsigned char addrWidth, unsigned char pipe)
{
	if(pipe >= 0 && pipe <=1)
		WriteRegBuf(nRF24L01P_WRITE_REG + nRF24L01P_RX_ADDR_P0 + pipe, pBuf, addrWidth);
	else
		setRXAddress(pBuf[0], addrWidth, pipe);
}//end setRXAddress


void nRF24L01P::setRXAddress(unsigned char pBuf, unsigned char addrWidth, unsigned char pipe)
{
	if(pipe >= 2 && pipe <=5)
		WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_RX_ADDR_P0 + pipe, pBuf);
}//end setRXAddress


void nRF24L01P::getTXAddress(unsigned char *pBuf)
{
	ReadRegBuf(nRF24L01P_TX_ADDR, pBuf, (ReadRegByte(nRF24L01P_SETUP_AW) & 0x03) + 2);
}//end getTXAddress


void nRF24L01P::getRXAddress(unsigned char *pBuf, unsigned char pipe)
{
	if(pipe >= 0 && pipe <= 1)
		ReadRegBuf(nRF24L01P_RX_ADDR_P0 + pipe, pBuf, (ReadRegByte(nRF24L01P_SETUP_AW) & 0x03) + 2);
	else if(pipe > 1 &&  pipe < 6)
	{
		ReadRegBuf(nRF24L01P_RX_ADDR_P1, pBuf, (ReadRegByte(nRF24L01P_SETUP_AW) & 0x03) + 2);
		pBuf[0] = ReadRegByte(nRF24L01P_RX_ADDR_P0 + pipe);
	}
}//end getTXAddress


void nRF24L01P::FlushTX(void)
{
	WriteRegByte(nRF24L01P_FLUSH_TX, 0);
}//end FlsuhTX


void nRF24L01P::FlushRX(void)
{
	WriteRegByte(nRF24L01P_FLUSH_RX, 0);
}//end FlsuhRX


void nRF24L01P::clearTxDs(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_STATUS, 0x2E);
}//end clearTxDs


void nRF24L01P::clearRxDr(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_STATUS, 0x4E);
}//end clearRxDr


void nRF24L01P::clearMaxRt(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_STATUS, 0x1E);
}//end clearMaxRt


void nRF24L01P::clearStatusReg(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_STATUS, 0x7E);
}//end clearStatusReg

unsigned char nRF24L01P::RxDR(void)
{
	return ((getStatus() & 0x40) >> 6);
}//end RxDR


unsigned char nRF24L01P::Available(void)
{
	unsigned char plWidth;

	if(isDynamicPDEnabled())
	{
		plWidth = ReadRegByte(nRF24L01P_R_RX_PL_WID);
	
		if(plWidth > 31)
			FlushRX();
	}//end if

	return (!isRxFIFOEmpty());
}//end Available


unsigned char nRF24L01P::MaxRt(void)
{
	unsigned char tmp;
	
	tmp = getStatus();
	tmp &= 0x10;
	tmp >>= 4;
	return tmp;
}//end MaxRt


unsigned char nRF24L01P::TxDs(void)
{
	return ((getStatus() & 0x20) >> 5);
}//end TxDs


unsigned char nRF24L01P::isTransmitted(void)
{
	return (TxDs());
}//end isTransmitted


unsigned char nRF24L01P::PacketLostCounter(void)
{
	return (ReadRegByte(nRF24L01P_OBSERVE_TX) >> 4);
}//end PocketLostCounter


void nRF24L01P::ClearPacketLostCounter(void)
{
	setChannel(getChannel());
}//end ClearPocketLostCounter


unsigned char nRF24L01P::RetransmittedPackets(void)
{
	return ((ReadRegByte(nRF24L01P_OBSERVE_TX) >> 4) & 0x0F);
}//end RetransmittedPockets


bool nRF24L01P::isTxFIFOFull(void)
{
	return ((ReadRegByte(nRF24L01P_FIFO_STATUS) & 0x20) >> 5);
}//end isTxFIFOFull


bool nRF24L01P::isTxFIFOEmpty(void)
{
	return ((ReadRegByte(nRF24L01P_FIFO_STATUS) & 0x10) >> 4);
}//end isTxFIFOEmpty


bool nRF24L01P::isRxFIFOFull(void)
{
	return ((ReadRegByte(nRF24L01P_FIFO_STATUS) & 0x02) >> 1);
}//end isRxFIFOFull


bool nRF24L01P::isRxFIFOEmpty(void)
{
	return (ReadRegByte(nRF24L01P_FIFO_STATUS) & 0x01);
}//end isRxFIFOEmpty


void nRF24L01P::EnableDynamicPayloadLength(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_FEATURE, 0x04);
}//end EnableDynamicPayloadLength


void nRF24L01P::DisableDynamicPayloadLength(void)
{
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_FEATURE, 0x00);
}//end DisableDynamicPayloadLength


void nRF24L01P::DataPipeEnableDynamicPldLen(unsigned char pipe)
{
	if(pipe > 5)
		return;

	EnableDataPipe(pipe);
    EnableAutoAck(pipe);
	EnableDynamicPayloadLength();
	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_DYNPD, ReadRegByte(nRF24L01P_DYNPD) | (0x01 << pipe));
}//end DataPipeEnableDynamicPldLen


void nRF24L01P::DataPipeDisableDynamicPldLen(unsigned char pipe)
{
	if(pipe > 5)
		return;

	WriteRegByte(nRF24L01P_WRITE_REG + nRF24L01P_DYNPD, ReadRegByte(nRF24L01P_DYNPD) & (~(0x01 << pipe)));
}//end DataPipeDisableDynamicPldLen


bool nRF24L01P::isDynamicPDEnabled(void)
{
	return ((ReadRegByte(nRF24L01P_FEATURE) & 0x04) >> 2);
}//end isDynamicPDEnabled


unsigned char nRF24L01P::SPIWriteByte(unsigned char dat)
{
	unsigned char i;

	digitalWrite(__clkPin, LOW);

	for(i=0; i<8; i++)
	{
		//output the MSB of byte
		if(dat & 0x80)
			digitalWrite(__mosiPin, HIGH);
		else
			digitalWrite(__mosiPin, LOW);
		
		//shift next bit
		dat <<= 1;
		digitalWrite(__clkPin, HIGH);
		

		//Capture Current MISO Bit
		if(digitalRead(__misoPin) == 1)
			dat |= 0x01;

		digitalWrite(__clkPin, LOW);
	}//end for

	//return capture of MISO
	return dat;
}//end SPIWriteByte

unsigned char nRF24L01P::SPIReadByte(void)
{
	return SPIWriteByte(0);
}//end SPIReadByte
