/*
The Chinese University of Hong Kong

File    : nRF24L01P.h
Version : 1.0
Author  : LO Ka Wo, Stanley

Last Modification Date : 2011 August 14, 05:22am

Description :
	This library provides an interface between Arduino and nRF24L01+.
	This library is able to make nRF24L01+ operate in ShockBurst mode
	with fixed data/payload length or Dynamic Data/Payload length.
	HOWEVER, this library cannot let nRF24L01+ communicate with nRF2401
	device(older model of nRF24L01+). 
	
	You can redistribute it and/or modify it.
	
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
    DEALINGS IN THE SOFTWARE.
*/

#ifndef __NRF24L01P_H__
#define __NRF24L01P_H__

#if defined(ARDUINO) && ARDUINO >= 100
  #include "Arduino.h"
#else
  #include "WProgram.h"
#endif

//********************************************************************************************************************//
// SPI(nRF24L01) commands
#define nRF24L01P_READ_REG        0x00  // Define read command to register
#define nRF24L01P_WRITE_REG       0x20  // Define write command to register
#define nRF24L01P_RD_RX_PLOAD     0x61  // Define RX payload register address
#define nRF24L01P_WR_TX_PLOAD     0xA0  // Define TX payload register address
#define nRF24L01P_FLUSH_TX        0xE1  // Define flush TX register command
#define nRF24L01P_FLUSH_RX        0xE2  // Define flush RX register command
#define nRF24L01P_REUSE_TX_PL     0xE3  // Define reuse TX payload register command
#define nRF24L01P_R_RX_PL_WID	  0x60  // Define Read RX Payload width for the top register command
#define nRF24L01P_NOP             0xFF  // Define No Operation, might be used to read status register

//********************************************************************************************************************//
// SPI(nRF24L01) registers(addresses)
#define nRF24L01P_CONFIG          0x00  // 'Config' register address
#define nRF24L01P_EN_AA           0x01  // 'Enable Auto Acknowledgment' register address
#define nRF24L01P_EN_RXADDR       0x02  // 'Enabled RX addresses' register address
#define nRF24L01P_SETUP_AW        0x03  // 'Setup address width' register address
#define nRF24L01P_SETUP_RETR      0x04  // 'Setup Auto. Retrans' register address
#define nRF24L01P_RF_CH           0x05  // 'RF channel' register address
#define nRF24L01P_RF_SETUP        0x06  // 'RF setup' register address
#define nRF24L01P_STATUS          0x07  // 'Status' register address
#define nRF24L01P_OBSERVE_TX      0x08  // 'Observe TX' register address
#define nRF24L01P_CD              0x09  // 'Carrier Detect' register address
#define nRF24L01P_RX_ADDR_P0      0x0A  // 'RX address pipe0' register address
#define nRF24L01P_RX_ADDR_P1      0x0B  // 'RX address pipe1' register address
#define nRF24L01P_RX_ADDR_P2      0x0C  // 'RX address pipe2' register address
#define nRF24L01P_RX_ADDR_P3      0x0D  // 'RX address pipe3' register address
#define nRF24L01P_RX_ADDR_P4      0x0E  // 'RX address pipe4' register address
#define nRF24L01P_RX_ADDR_P5      0x0F  // 'RX address pipe5' register address
#define nRF24L01P_TX_ADDR         0x10  // 'TX address' register address
#define nRF24L01P_RX_PW_P0        0x11  // 'RX payload width, pipe0' register address
#define nRF24L01P_RX_PW_P1        0x12  // 'RX payload width, pipe1' register address
#define nRF24L01P_RX_PW_P2        0x13  // 'RX payload width, pipe2' register address
#define nRF24L01P_RX_PW_P3        0x14  // 'RX payload width, pipe3' register address
#define nRF24L01P_RX_PW_P4        0x15  // 'RX payload width, pipe4' register address
#define nRF24L01P_RX_PW_P5        0x16  // 'RX payload width, pipe5' register address
#define nRF24L01P_FIFO_STATUS     0x17  // 'FIFO Status Register' register address
#define nRF24L01P_DYNPD 		  0x1C  // 'Enable dynamic payload length' register address
#define nRF24L01P_FEATURE         0x1D  // 'Feature' register address


#define TX 0                            // Define a flag for Transmitter
#define RX 1                            // Define a flag for Receiver
#define SpeedAirRate_1MBPS   0          // Define a constant for 1Mbps Speed Aire Rate
#define SpeedAirRate_2MBPS   1          // Define a constant for 2Mbps Speed Aire Rate
#define SpeedAirRate_250KBPS 2          // Define a constant for 250Kbps Speed Aire Rate


class nRF24L01P
{
	public:
		//constructor
		nRF24L01P(int cePin,   int csnPin, 
				  int clkPin,  int mosiPin,
				  int misoPin);
		
		//Default Shock Burst Setting 
		//Set Device to Power UP and Switch to RX Mode
		//Enable RX Address Data Pipe 0 and 1
		//Set Rx/Tx Address Width 5 bytes
		//Set Retransmission Delay 4ms
		//Set Retransmission Count 15
		//set Operating Channel 23
		//Set Speed Air Rate 2Mbps
		//set RF Power 0dBm
		//Flush TX and RX FIFO Buffer
		//Clear Status Register
		void DefaultShockBurstSetting(void);

		//Initial Setting as same as Default Shock Burst Setting
		//Enable Data Pipe 0 and 1 Operates in dynamic payload mode
		void DefaultDynamicPayloadSetting(void);

		//Power UP
		void PowerUP(void);

		//Power Down(void);
		void PowerDown(void);

		//Disable RX_DR Interrupt
		void DisableRxDrInterrupt(void);

		//Enable RX_DR Interrupt
		void EnableRxDrInterrupt(void);

		//Disable TX_DS Interrupt
		void DisableTxDsInterrupt(void);

		//EnableTX_DS Interrupt
		void EnableTxDsInterrupt(void);

		//Disable MASK_MAX_RT Interrupt
		void DisableMaxRtInterrupt(void);

		//Enable MASK_MAX_RT Interrupt
		void EnableMaxRtInterrupt(void);

		//Disable CRC
		void DisableCRC(void);

		//Enable CRC
		//Parameter: scheme : 0 for 1 byte, 1 for 2 bytes
		void EnableCRC(unsigned char scheme);

		//Enable Auto Acknowledgement
		//Parameter: pipe: auto acknowledgement data pipe no. to be enable 
		void EnableAutoAck(unsigned char pipe);


		//Disable Auto Acknowledgement
		//Parameter: pipe: auto acknowledgement data pipe no. to be disable 
		void DisableAutoAck(unsigned char pipe);

		//Disable Data Pipe
		//pipe: Data Pipe Number
		//      which is from 0 to 5
		void DisableDataPipe(unsigned char pipe);
		
		//Enable Data Pipe
		//pipe: Data Pipe Number
		//      which is from 0 to 5
		void EnableDataPipe(unsigned char pipe);

		//Disable Data Pipe
		//pipe: Data Pipe Number
		//      which is from 0 to 5
		void DisableRXAddr(unsigned char pipe);

		//Enable Data Pipe
		//pipe: Data Pipe Number
		//      which is from 0 to 5
		void EnableRXAddr(unsigned char pipe);

		//Set RX/TX Address Width
		//Parameter: AddrWidth: from 3 to 5
		void setAddressWidth(unsigned char AddrWidth);

		//Get RX/TX Address Width
		//Return: number of bytes of address
		unsigned char getAddressWidth(void);

		//Set Automatic Retransmission Delay
		//Parameter: AutoRD : from 0 to 15
		//           Auto Re-transmission Delay: (AutoRD + 1) * 250us
		void setAutoRD(unsigned char AutoRD);

		//Set Automatic Retransmission Count 
		//Parameter: AutoCount: from 0 to 15
		void setAutoReCount(unsigned char AutoCount);

		//Set Speed Aire Rate
		//Parameter: Speed: from 0 to 2, default=1
		//                  0 for 1Mbps, 1 for 2Mbps, 2 for 250Kbps
		//                  or use:
        //                         SpeedAirRate_1MBPS
		//                         SpeedAirRate_2MBPS
        //                         SpeedAirRate_250KBPS
		void setSpeedAirRate(unsigned char Speed);

		//Set RF Power
		//Parameter: PWR: from 0 to 3, default=3
		//                0 for -18dBm
		//                1 for -12dBm
		//                2 for -6dBm
		//                3 for 0dBm
		void setRFPower(unsigned char PWR);

		//Write Register
		void WriteRegByte(unsigned char reg, unsigned char value); 

		//Read Register
		unsigned char ReadRegByte(unsigned char reg); 

		//Write Buffer to a register
		void WriteRegBuf(unsigned char reg, unsigned char *buf, char bytes);

		//Read Brffer from a register
		void ReadRegBuf(unsigned char reg, unsigned char *buf, char bytes);

		//Set Mode
		//Parameter: Mode : TX for transmitter
		//                  RX for receiver
		void setMode(char mode);

		//Transmit Data
		//Parameters: *pBuf byte to be sent
		//            plWidth : no. of bytes in pBuf will be sent
		void Transmit(unsigned char *pBuf, unsigned char plWidth);
		void Transmit(char *pBuf, unsigned char plWidth);

		//Write Transmit Payload
		//Parameters: *pBuf byte to be written 
		//            plWidth : no. of bytes in pBuf will be written 
		void WriteTXPayload(unsigned char *pBuf, unsigned char plWidth);
		
		//Set channel
		//Parameter: RF_Channel : from 0 to 125
		void setChannel(unsigned char RF_Channel);

		//Get Channel
		//Return RF Channel From 0 to 125
		unsigned char getChannel(void);

		//Get Status Register
		unsigned char getStatus(void);

		//Get Received Payload Width of a specific Data Pipe
		//Parameters: pipe: Data Pipe Number
		//Return: Number of bytes are received in specific data pipe
		unsigned char getRXPayloadWidth(unsigned char pipe);

		//Set RX Payload Width of a specific Data Pipe
		//Parameter: pipe: pipe no. which is from 0 to 5
		//			 plWidth: Payload width which is from 1 to 32
		void setRXPayloadWidth(unsigned char pipe, unsigned char plWidth);

		//Get Current Pipe Num
		//return a pipe num which data is ready for receiving
		unsigned char getCurrentPipeNum(void);

		//Get RX Payload / Get Received Data
		//Notes: This function is same as getData
		void getRXPload(unsigned char *pBuf);

		//Get RX Payload / Get Received Data
		//Parameters: *pipe : the data pipe num will be returned
		//			  *plWidth : the Width of payload will be returned
		//Notes: This function is same as getData
		void getRXPload(unsigned char *pBuf, unsigned char *pipe, unsigned char *plWidth);
		
		//Get Received Data
		void getData(unsigned char *pBuf);
		void getData(char *pBuf);
		
		//Get Received Data
		//Parameters: *pipe : the data pipe num will be returned
		//			  *plWidth : the Width of payload will be returned
		void getData(unsigned char *pBuf, unsigned char *pipe, unsigned char *plWidth);
		void getData(char *pBuf, unsigned char *pipe, unsigned char *plWidth);

		//Get FIFO Status
		unsigned char getFIFO(void);
		
		//Set TX Address
		//Parameters: *pBuf: Address
		//			  addrWidth: no. of bytes of *pBuf
		void setTXAddress(unsigned char *pBuf, unsigned char addrWidth);

		//Set TX Address
		//Parameters: *pBuf: Address
		//			  addrWidth: no. of bytes of *pBuf
		//			  pipe: no. of pipe which is from 0 to 5
		void setRXAddress(unsigned char *pBuf, unsigned char addrWidth, unsigned char pipe);
		void setRXAddress(unsigned char pBuf, unsigned char addrWidth, unsigned char pipe);

		//Get TX Address
		//Parameters: *pBuf: The TX Address will be returned.
		void getTXAddress(unsigned char *pBuf);

		//Get RX Address
		//Parameters: *pBuf: The RX Address will be returned.
		//            pipe:  select the data pipe channel
		void getRXAddress(unsigned char *pBuf, unsigned char pipe);

		//Flush TX FIFO Buffer
		void FlushTX(void);

		//Flush RX FIFO Buffer
		void FlushRX(void);

		//Clear TX_DS Bit
		void clearTxDs(void);

		//Clear RX_DR Bit
		void clearRxDr(void);

		//Clear Max_RT Bit
		void clearMaxRt(void);

		//Clear Status Register
		void clearStatusReg(void);

		//RxDR
		//return 1: Data received is ready
		//       0: No data is received.
		unsigned char RxDR(void);

		//Available
		//return 1: Data received is ready
		//       0: No data is received.
		unsigned char Available(void);

		//MaxRt
		//return 1: Maximum number of TX retransmits interrupt
		//       0: otherwise
		//Notes: If 1 is returned, it is meant that the transmission
		//       is fialed. The data cannot be sent to target device.
		unsigned char MaxRt(void);

		//TxDs
		//return 1: Data transmitted successfully.
		//       0: otherwise
		unsigned char TxDs(void);

		//Transmitted
		//return 1: Data transmitted successfully
		//       0: otherwise
		unsigned char isTransmitted(void);

		//Pocker Lost
		//return: how many packets are lost
		//notes: the maximum number of packet lost is 15
		//       overflow will be occurred when it is greater
		//       than 15
		unsigned char PacketLostCounter(void);

		//Clear Packet Lost Counter
		void ClearPacketLostCounter(void);

		//Return number of Pockets Retransmitted
		unsigned char RetransmittedPackets(void);	

		//Check TX FIFO whether is Full
		//return: true for TX FIFO Full
		//        false for Available locations in TX FIFO
		bool isTxFIFOFull(void);

		//Check TX FIFO whether is Empty
		//return: true for TX FIFO Empty
		//        false for TX FIFO not Empty
		bool isTxFIFOEmpty(void);

		//Check RX FIFO whether is Full
		//return: true for RX FIFO Full
		//        false for Available locations in RX FIFO
		bool isRxFIFOFull(void);

		//Check RX FIFO whether is Empty
		//return: true for RX FIFO Empty
		//        false for RX FIFO not Empty
		bool isRxFIFOEmpty(void);

		//Enable Dynamic Payload Length Function
		void EnableDynamicPayloadLength(void);

		//Disable Dynamic Payload Length Function
		void DisableDynamicPayloadLength(void);

		//Data Pipe Enable Dynamic Payload Length
		//Parameters: pipe: data pipe to be enable the Dynamic Payload Length
		void DataPipeEnableDynamicPldLen(unsigned char pipe);

		//Data Pipe Disable Dynamic Payload Length
		//Parameters: pipe: data pipe to be disable the Dynamic Payload Length
		void DataPipeDisableDynamicPldLen(unsigned char pipe);
		
		//is Dynamic Payload Enabled
		//return: check whether the dynamic payload length function is enabled
		bool isDynamicPDEnabled(void);

	private:
		int __csnPin;
		int __cePin;
		int __clkPin;
		int __mosiPin;
		int __misoPin;

		unsigned char SPIWriteByte(unsigned char dat);
		unsigned char SPIReadByte(void);
};
#endif
